from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, pattern_match

awss3_server_access_insecure_tests: list[RuleTest] = [
    RuleTest(
        name="Secure Access to S3 Bucket",
        expected_result=False,
        log={
            "bucketowner": "f16a9e81a6589df1c902c86f7982fd14a88787db",
            "bucket": "cloudtrail",
            "time": "2020-02-14 00:53:48.000000000",
            "remoteip": "127.0.0.1",
            "requester": "arn:aws:sts::123456789012:assumed-role/eagle/regionalDeliverySession",
            "requestid": "101B7403B9828743",
            "operation": "REST.PUT.OBJECT",
            "key": "AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz",
            "requesturi": "PUT /AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz HTTP/1.1",
            "httpstatus": 200,
            "objectsize": 747,
            "totaltime": 110,
            "turnaroundtime": 20,
            "useragent": "aws-internal/3 aws-sdk-java/1.11.714 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
            "hostid": "neRpT/AXRsS3LMBqq/wND59opwPRWWKn7F6evEhdbS99me5fyIXpVI/MMIn6ECgU1YZAqwuF8Bw=",
            "signatureversion": "SigV4",
            "ciphersuite": "ECDHE-RSA-AES128-SHA",
            "authenticationtype": "AuthHeader",
            "hostheader": "cloudtrail.s3.us-east-1.amazonaws.com",
            "tlsVersion": "TLSv1.2",
            "p_log_type": "AWS.S3ServerAccess",
            "p_row_id": "8855aa99ff77abc8dcb0e36e0a",
            "p_event_time": "2020-02-14 00:53:48.000000000",
            "p_any_ip_addresses": ["55.99.86.234"],
            "p_any_aws_arns": ["arn:aws:sts::123456789012:assumed-role/eagle/regionalDeliverySession"],
        },
    ),
    RuleTest(
        name="Delete Marker Call",
        expected_result=False,
        log={
            "bucketowner": "06c722119dedc1896ef",
            "bucket": "panther-yyykkj4jj66e",
            "time": "2020-05-21 07:05:13.000000000",
            "requester": "AmazonS3",
            "requestid": "1366A0B06E7A7728",
            "operation": "S3.CREATE.DELETEMARKER",
            "key": "test/083ec760-bbbb-4444-8888-185614f4b0fc.csv.metadata",
            "versionid": "mNwtwD6vrqwx11g9kSpb2MDY",
            "hostid": "KxrLgSKGXXiKBBhTYbks6XeL1juvDqx+OBHflvk",
            "p_log_type": "AWS.S3ServerAccess",
            "p_row_id": "4ec7d72e6f5392c2c7c1b6e302ee01",
            "p_event_time": "2020-05-21 07:05:13.000000000",
            "p_parse_time": "2020-05-21 08:19:50.085391216",
        },
    ),
    RuleTest(
        name="Insecure Access to S3 Bucket",
        expected_result=True,
        log={
            "authenticationtype": "AuthHeader",
            "bucket": "cloudtrail",
            "bucketowner": "f16a9e81a6589df1c902c86f7982fd14a88787db",
            "hostheader": "cloudtrail.s3.us-east-1.amazonaws.com",
            "hostid": "neRpT/AXRsS3LMBqq/wND59opwPRWWKn7F6evEhdbS99me5fyIXpVI/MMIn6ECgU1YZAqwuF8Bw=",
            "httpstatus": 200,
            "key": "AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz",
            "objectsize": 747,
            "operation": "REST.PUT.OBJECT",
            "p_any_aws_arns": ["arn:aws:sts::123456789012:assumed-role/eagle/regionalDeliverySession"],
            "p_any_ip_addresses": ["55.99.86.234"],
            "p_event_time": "2020-02-14 00:53:48.000000000",
            "p_log_type": "AWS.S3ServerAccess",
            "p_row_id": "8855aa99ff77abc8dcb0e36e0a",
            "remoteip": "127.0.0.1",
            "requester": "arn:aws:sts::123456789012:assumed-role/eagle/regionalDeliverySession",
            "requestid": "101B7403B9828743",
            "requesturi": "PUT /AWSLogs/o-wwwwwwgggg/234567890123/CloudTrail-Digest/ca-central-1/2020/02/14/234567890123_CloudTrail-Digest_ca-central-1_POrgTrail_us-east-1_20200214T001007Z.json.gz HTTP/1.1",
            "signatureversion": "SigV4",
            "time": "2020-02-14 00:53:48.000000000",
            "totaltime": 110,
            "turnaroundtime": 20,
            "useragent": "aws-internal/3 aws-sdk-java/1.11.714 Linux/4.9.184-0.1.ac.235.83.329.metal1.x86_64 OpenJDK_64-Bit_Server_VM/25.242-b08 java/1.8.0_242 vendor/Oracle_Corporation",
        },
    ),
]


@panther_managed
class AWSS3ServerAccessInsecure(Rule):
    id = "AWS.S3.ServerAccess.Insecure-prototype"
    display_name = "AWS S3 Insecure Access"
    dedup_period_minutes = 720
    log_types = [LogType.AWS_S3_SERVER_ACCESS]
    tags = ["AWS", "Configuration Required", "Security Control", "Collection:Data From Cloud Storage Object"]
    reports = {"MITRE ATT&CK": ["TA0009:T1530"]}
    default_severity = Severity.LOW
    default_description = (
        "Checks if HTTP (unencrypted) was used to access objects in an S3 bucket, as opposed to HTTPS (encrypted).\n"
    )
    default_runbook = "Add a condition on the S3 bucket policy that denies access via http.\n"
    default_reference = "https://aws.amazon.com/premiumsupport/knowledge-center/s3-bucket-policy-for-config-rule/"
    summary_attributes = [
        "bucket",
        "key",
        "operation",
        "userAgent",
        "remoteip",
        "requester",
        "p_any_aws_arns",
        "p_any_aws_account_ids",
    ]
    tests = awss3_server_access_insecure_tests

    def rule(self, event):
        return pattern_match(event.get("operation", ""), "REST.*.OBJECT") and (
            not event.get("ciphersuite") or not event.get("tlsVersion")
        )

    def title(self, event):
        return f"Insecure access to S3 Bucket [{event.get('bucket', '<UNKNOWN_BUCKET>')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_misc_settings_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk Feature Flags changed",
        expected_result=True,
        log={
            "created": "2023-04-11 23:32:14.173",
            "event": "group.feature_flags.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk User Invite Revoke",
        expected_result=False,
        log={
            "content": {},
            "created": "2023-04-11 23:32:13.248",
            "event": "org.user.invite.revoke",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
]


@panther_managed
class SnykMiscSettings(Rule):
    id = "Snyk.Misc.Settings-prototype"
    display_name = "Snyk Miscellaneous Settings"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_reference = "https://docs.snyk.io/snyk-admin/manage-settings"
    default_severity = Severity.LOW
    default_description = "Detects when Snyk settings that lack a clear security impact are changed\n"
    summary_attributes = ["event"]
    tests = snyk_misc_settings_tests
    ACTIONS = ["group.cloud_config.settings.edit", "group.feature_flags.edit"]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            operation = ".".join(action.split(".")[1:]).title()
        return f"Snyk: [{group_or_org}] Setting [{operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        return snyk_alert_context(event)

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"



"""Used to build client with assumed audit role."""

import os

import boto3

PANTHER_MASTER_REGION = os.environ.get("AWS_REGION")


def build_client(resource, service, region=None):
    """Function builds resource client that assumes Panther audit role"""
    account = resource["AccountId"]
    role_arn = f"arn:aws:iam::{account}:role/PantherAuditRole-{PANTHER_MASTER_REGION}"

    fips_enabled = os.getenv("ENABLE_FIPS", "").lower() == "true"
    fips_suffix = "-fips." + PANTHER_MASTER_REGION + ".amazonaws.com"

    sts_connection = boto3.client("sts", endpoint_url="https://sts" + fips_suffix if fips_enabled else None)

    acct_b = sts_connection.assume_role(RoleArn=role_arn, RoleSessionName="lambda_assume_audit_role")
    access_key = acct_b["Credentials"]["AccessKeyId"]
    secret_key = acct_b["Credentials"]["SecretAccessKey"]
    session_token = acct_b["Credentials"]["SessionToken"]
    # create service client using the assumed role credentials, e.g. S3
    if region is None:
        client = boto3.client(
            service,
            aws_access_key_id=access_key,
            aws_secret_access_key=secret_key,
            aws_session_token=session_token,
        )
    else:
        client = boto3.client(
            service,
            region="fips-" + region if fips_enabled and region else region,
            endpoint_url="https://sts" + fips_suffix if fips_enabled else None,
            aws_access_key_id=access_key,
            aws_secret_access_key=secret_key,
            aws_session_token=session_token,
        )
    return client



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_cloud_storage_buckets_modified_or_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="other event",
        expected_result=False,
        log={
            "insertid": "ezyd47c12y",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-09 16:41:30.524",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-09 16:44:14.617",
            "p_row_id": "1234567909689348911",
            "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "test@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "logging.sinks.update",
                        "resource": "projects/gcp-project1/sinks/log-sink",
                        "resourceAttributes": {
                            "name": "projects/gcp-project1/sinks/log-sink",
                            "service": "logging.googleapis.com",
                        },
                    },
                ],
                "methodName": "google.logging.v2.ConfigServiceV2.UpdateSink",
                "request": {
                    "@type": "type.googleapis.com/google.logging.v2.UpdateSinkRequest",
                    "sink": {
                        "destination": "pubsub.googleapis.com/projects/gcp-project1/topics/gcp-topic1",
                        "exclusions": [{"filter": "protoPayload.serviceName = 'k8s.io", "name": "excludek8s"}],
                        "name": "log-sink",
                        "writerIdentity": "serviceAccount:p197946410614-915152@gcp-sa-logging.iam.gserviceaccount.com",
                    },
                    "sinkName": "projects/gcp-project1/sinks/log-sink",
                    "uniqueWriterIdentity": True,
                    "updateMask": "exclusions",
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-03-09T16:41:30.540045105Z"},
                },
                "resourceName": "projects/gcp-project1/sinks/log-sink",
                "serviceName": "logging.googleapis.com",
                "status": {},
            },
            "receivetimestamp": "2023-03-09 16:41:32.21",
            "resource": {
                "labels": {"destination": "", "name": "log-sink", "project_id": "gcp-project1"},
                "type": "logging_sink",
            },
            "severity": "NOTICE",
            "timestamp": "2023-03-09 16:41:30.524",
        },
    ),
    RuleTest(
        name="bucket update",
        expected_result=True,
        log={
            "insertId": "asdf1234asdfg",
            "logName": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-09 10:05:23.603",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-09 10:07:14.731",
            "p_row_id": "7ad218d42253b7e6f78cc0ed1635",
            "p_source_id": "4fc88a5a-2d51-4279-9c4a-08fa7cc52566",
            "p_source_label": "gcplogsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "storage.buckets.update",
                        "resource": "projects/_/buckets/my-bucket",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "storage.buckets.update",
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "apitools Python/3.9.11 gsutil/5.11 (darwin) analytics/enabled interactive/True command/notification google-cloud-sdk/394.0.0,gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-03-09T10:05:23.610372568Z"},
                },
                "resourceName": "projects/_/buckets/my-bucket",
                "serviceName": "storage.googleapis.com",
                "status": {},
            },
            "receiveTimestamp": "2023-03-09 10:05:25.146",
            "resource": {
                "labels": {"bucket_name": "my-bucket", "location": "us-east1", "project_id": "gcp-project1"},
                "type": "gcs_bucket",
            },
            "severity": "NOTICE",
            "timestamp": "2023-03-09 10:05:23.603",
        },
    ),
]


@panther_managed
class GCPCloudStorageBucketsModifiedOrDeleted(Rule):
    default_description = "Detects GCP cloud storage bucket updates and deletes."
    display_name = "GCP Cloud Storage Buckets Modified Or Deleted"
    default_reference = "https://cloud.google.com/storage/docs/buckets"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.Cloud.Storage.Buckets.Modified.Or.Deleted-prototype"
    tests = gcp_cloud_storage_buckets_modified_or_deleted_tests
    BUCKET_OPERATIONS = ["storage.buckets.delete", "storage.buckets.update"]

    def rule(self, event):
        return all(
            [
                deep_get(event, "protoPayload", "serviceName", default="") == "storage.googleapis.com",
                deep_get(event, "protoPayload", "methodName", default="") in self.BUCKET_OPERATIONS,
            ],
        )

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        bucket = deep_get(event, "resource", "labels", "bucket_name", default="<BUCKET_NOT_FOUND>")
        return f"GCP: [{actor}] performed a [{operation}] on bucket [{bucket}] in project [{project}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

decoy_dynamo_db_accessed_tests: list[RuleTest] = [
    RuleTest(
        name="DynamoDB-Decoy-Accessed",
        expected_result=True,
        log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "Scan",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "RemoteIpDetails": {
                        "City": {},
                        "Country": {},
                        "GeoLocation": {},
                        "IpAddressV4": "11.111.11.111",
                        "Organization": {},
                    },
                    "ServiceName": "dynamodb.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 22:53:24.000000000",
            "Description": "Private decoy DynamoDB table arn:aws:dynamodb:us-east-1:123456789012:table/Panther-DataTable was accessed by arn:aws:iam::123456789012:user/tester. This DynamoDB table has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {"Severity": {"Label": "HIGH", "Normalized": 70}, "Types": ["Unusual Behaviors"]},
            "GeneratorId": "dynamodb.amazonaws.com",
            "Id": "1abc2de3-69ea-4e15-91c6-27eb4a07bd21",
            "ProcessedAt": "2024-05-24T22:53:41.884Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd234-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default/1abc2de3-69ea-4e15-91c6-27eb4a07bd21",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "<id>",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsDynamoDbTable",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABC9ONWNS3155VIEJC8U",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABC9ONWNS3155VIEJC8U:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {"CreationDate": "2024-05-24T22:32:38Z", "MfaAuthenticated": False},
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:user/tester",
                                    "PrincipalId": "tester",
                                    "Type": "Role",
                                    "UserName": "user/tester",
                                },
                            },
                        },
                    },
                    "Id": "ABC9ONWNS3155VIEJC8U",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "arn:aws:iam::123456789012:user/tester",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Suspicious activity detected accessing private decoy DynamoDB table arn:aws:dynamodb:us-east-1:123456789012:table/Panther-DataTable",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 22:53:24.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_ip_addresses": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 22:53:24.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 22:55:04.312964001",
            "p_row_id": "8e1c8ebd709fb49e9eb5a1c61ff1a303",
            "p_schema_version": 0,
            "p_source_id": "e29fd64f-53d9-43ab-92ca-575a8af289e6",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
    RuleTest(
        name="DynamoDB-Decoy-Not-Accessed",
        expected_result=False,
        log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "DescribeEndpoints",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "RemoteIpDetails": {
                        "City": {},
                        "Country": {},
                        "GeoLocation": {},
                        "IpAddressV4": "11.111.11.111",
                        "Organization": {},
                    },
                    "ServiceName": "dynamodb.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 22:53:24.000000000",
            "Description": "Private decoy DynamoDB table arn:aws:dynamodb:us-east-1:123456789012:table/Panther-DataTable was not accessed by arn:aws:iam::123456789012:user/tester. This DynamoDB table has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {"Severity": {"Label": "HIGH", "Normalized": 70}, "Types": ["Unusual Behaviors"]},
            "GeneratorId": "dynamodb.amazonaws.com",
            "Id": "1abc2de3-69ea-4e15-91c6-27eb4a07bd21",
            "ProcessedAt": "2024-05-24T22:53:41.884Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd234-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default/1abc2de3-69ea-4e15-91c6-27eb4a07bd21",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "<id>",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsDynamoDbTable",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABC9ONWNS3155VIEJC8U",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABC9ONWNS3155VIEJC8U:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {"CreationDate": "2024-05-24T22:32:38Z", "MfaAuthenticated": False},
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:user/tester",
                                    "PrincipalId": "tester",
                                    "Type": "Role",
                                    "UserName": "user/tester",
                                },
                            },
                        },
                    },
                    "Id": "ABC9ONWNS3155VIEJC8U",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "arn:aws:iam::123456789012:user/tester",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Non-Suspicious activity detected accessing private decoy DynamoDB table arn:aws:dynamodb:us-east-1:123456789012:table/Panther-DataTable",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 22:53:24.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_ip_addresses": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 22:53:24.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 22:55:04.312964001",
            "p_row_id": "8e1c8ebd709fb49e9eb5a1c61ff1a303",
            "p_schema_version": 0,
            "p_source_id": "e29fd64f-53d9-43ab-92ca-575a8af289e6",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
]


@panther_managed
class DecoyDynamoDBAccessed(Rule):
    id = "Decoy.DynamoDB.Accessed-prototype"
    display_name = "Decoy DynamoDB Accessed"
    enabled = False
    log_types = [LogType.AWS_SECURITY_FINDING_FORMAT]
    default_severity = Severity.HIGH
    default_description = "Actor accessed Decoy DynamoDB"
    default_reference = "https://aws.amazon.com/blogs/security/how-to-detect-suspicious-activity-in-your-aws-account-by-using-private-decoy-resources/"
    inline_filters = [{"All": []}]
    tests = decoy_dynamo_db_accessed_tests

    def rule(self, event):
        # List of suspicious API events
        # NOTE: There may be more API events that's not listed
        suspicious_api_events = [
            "BatchExecuteStatement",
            "BatchGetItem",
            "BatchWriteItem",
            "DeleteItem",
            "ExecuteStatement",
            "ExecuteTransaction",
            "GetItem",
            "PutItem",
            "Query",
            "Scan",
            "TransactGetItems",
            "TransactWriteItems",
            "UpdateItem",
        ]
        # Return True if the API value is in the list of suspicious API events
        if event["GeneratorId"] == "dynamodb.amazonaws.com":
            # Extract the API value from the event
            api_value = event["Action"]["AwsApiCallAction"]["Api"]
            return api_value in suspicious_api_events
        return False

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method will act as dedup string.
        # NOTE: Not sure if the offending actor Id will always be in the 0th index of Resources
        # It's possible to just return the Title as a whole string
        secret = event["Resources"][0]["Id"]
        return f"Suspicious activity detected accessing private decoy DynamoDB table {secret}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import box_parse_additional_details, deep_get

box_shield_anomalous_download_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": {'"key": "value"': None},
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="Anomalous Download Event",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Anomalous Download","risk_score":77,"alert_summary":{"description":"Significant increase in download content week over week, 9999% (50.00 MB) more than last week."}}}',
            "created_by": {"id": "12345678", "type": "user", "login": "bob@example", "name": "Bob Cat"},
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user", "login": "bob@example", "name": "Bob Cat"},
        },
    ),
]


@panther_managed
class BoxShieldAnomalousDownload(Rule):
    id = "Box.Shield.Anomalous.Download-prototype"
    display_name = "Box Shield Detected Anomalous Download Activity"
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Exfiltration:Exfiltration Over Web Service"]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.HIGH
    default_description = "A user's download activity has altered significantly.\n"
    default_reference = "https://developer.box.com/guides/events/shield-alert-events/"
    default_runbook = "Investigate whether this was triggered by expected user download activity.\n"
    summary_attributes = ["event_type", "ip_address"]
    tests = box_shield_anomalous_download_tests

    def rule(self, event):
        if event.get("event_type") != "SHIELD_ALERT":
            return False
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        if alert_details.get("rule_category", "") == "Anomalous Download":
            if alert_details.get("risk_score", 0) > 50:
                return True
        return False

    def title(self, event):
        details = box_parse_additional_details(event)
        description = deep_get(details, "shield_alert", "alert_summary", "description")
        if description:
            return description
        return f"Anomalous download activity triggered by user [{deep_get(event, 'created_by', 'name', default='<UNKNOWN_USER>')}]."



from pypanther import LogType, Rule, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context


@panther_managed
class NotionPagePermsAPIPermsChanged(Rule):
    display_name = "Notion Page API Permissions Changed"
    id = "Notion.PagePerms.APIPermsChanged-prototype"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Data Security", "Unapproved 3rd Party Apps"]
    default_severity = Severity.LOW
    default_description = "A new API integration was added to a Notion page, or it's permissions were changed."
    default_runbook = "Potential information exposure - review the shared page and rectify if needed."
    default_reference = "https://www.notion.so/help/sharing-and-permissions"
    # These event types correspond to users adding or editing the default role on a public page
    event_types = ("page.permissions.integration_role_added", "page.permissions.integration_role_updated")

    def rule(self, event):
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") in self.event_types

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        page_id = event.deep_get("event", "details", "target", "page_id", default="<NO_PAGE_ID_FOUND>")
        return f"Notion User [{user}] added an integration to page [{page_id}]."

    def alert_context(self, event):
        context = notion_alert_context(event)
        page_id = event.deep_get("event", "details", "target", "page_id", default="<NO_PAGE_ID_FOUND>")
        context["page_id"] = page_id
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context

mongo_d_borg_membership_restriction_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Restriction disabled",
        expected_result=True,
        log={
            "created": "2024-04-03 15:03:51.000000000",
            "currentValue": {},
            "eventTypeName": "ORG_PUBLIC_API_ACCESS_LIST_NOT_REQUIRED",
            "id": "alert_id",
            "isGlobalAdmin": False,
            "orgId": "some_org_id",
            "remoteAddress": "1.2.3.4",
            "userId": "user_id",
            "username": "some_user@company.com",
        },
    ),
    RuleTest(
        name="Restriction enabled",
        expected_result=False,
        log={
            "created": "2024-04-03 15:03:51.000000000",
            "currentValue": {},
            "eventTypeName": "ORG_PUBLIC_API_ACCESS_LIST_REQUIRED",
            "id": "alert_id",
            "isGlobalAdmin": False,
            "orgId": "some_org_id",
            "remoteAddress": "1.2.3.4",
            "userId": "user_id",
            "username": "some_user@company.com",
        },
    ),
    RuleTest(
        name="Other activity",
        expected_result=False,
        log={
            "alertConfigId": "alert_id",
            "created": "2024-04-01 11:58:52.000000000",
            "currentValue": {},
            "eventTypeName": "ALERT_CONFIG_DELETED_AUDIT",
            "id": "alert_id",
            "isGlobalAdmin": False,
            "links": [],
            "orgId": "some_org_id",
            "remoteAddress": "1.2.3.4",
            "userId": "user_id",
            "username": "some_user@company.com",
        },
    ),
]


@panther_managed
class MongoDBorgMembershipRestrictionDisabled(Rule):
    default_description = "You can configure Atlas to require API access lists at the organization level. When you enable IP access list for the Atlas Administration API, all API calls in that organization must originate from a valid entry in the associated Atlas Administration API key access list. This rule detects when IP access list is disabled"
    display_name = "MongoDB org membership restriction disabled"
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.org.Membership.Restriction.Disabled-prototype"
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["T1556"]}
    default_reference = "https://www.mongodb.com/docs/atlas/tutorial/manage-organizations/"
    default_runbook = (
        "Check if this activity is legitimate. If not, re-enable IP access list for the Atlas Administration API"
    )
    tests = mongo_d_borg_membership_restriction_disabled_tests

    def rule(self, event):
        return event.deep_get("eventTypeName", default="") == "ORG_PUBLIC_API_ACCESS_LIST_NOT_REQUIRED"

    def title(self, event):
        user = event.deep_get("username", default="<USER_NOT_FOUND>")
        return f"MongoDB: [{user}] has disabled IP access list for the Atlas Administration API"

    def alert_context(self, event):
        return mongodb_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context

mongo_db_access_allowed_from_anywhere_tests: list[RuleTest] = [
    RuleTest(
        name="Allowed access from anywhere",
        expected_result=True,
        log={
            "created": "2024-04-03 11:13:04.000000000",
            "currentValue": {},
            "eventTypeName": "NETWORK_PERMISSION_ENTRY_ADDED",
            "groupId": "some_group_id",
            "id": "123abc",
            "isGlobalAdmin": False,
            "remoteAddress": "1.2.3.4",
            "userId": "123abc",
            "username": "some_user@company.com",
            "whitelistEntry": "0.0.0.0/0",
        },
    ),
    RuleTest(
        name="Allowed access from specific ip",
        expected_result=False,
        log={
            "created": "2024-04-03 11:13:04.000000000",
            "currentValue": {},
            "eventTypeName": "NETWORK_PERMISSION_ENTRY_ADDED",
            "groupId": "some_group_id",
            "id": "123abc",
            "isGlobalAdmin": False,
            "remoteAddress": "1.2.3.4",
            "userId": "123abc",
            "username": "some_user@company.com",
            "whitelistEntry": "1.2.3.4/32",
        },
    ),
]


@panther_managed
class MongoDBAccessAllowedFromAnywhere(Rule):
    default_description = "Atlas only allows client connections to the database deployment from entries in the project's IP access list. This rule detects when 0.0.0.0/0 is added to that list, which allows access from anywhere."
    display_name = "MongoDB access allowed from anywhere"
    log_types = [LogType.MONGODB_PROJECT_EVENT]
    id = "MongoDB.Access.Allowed.From.Anywhere-prototype"
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["T1021"]}
    default_reference = "https://www.mongodb.com/docs/atlas/security/ip-access-list/"
    default_runbook = "Check if this activity was legitimate. If not, delete 0.0.0.0/0 from the list of allowed ips."
    tests = mongo_db_access_allowed_from_anywhere_tests

    def rule(self, event):
        if (
            event.deep_get("eventTypeName", default="") == "NETWORK_PERMISSION_ENTRY_ADDED"
            and event.deep_get("whitelistEntry", default="") == "0.0.0.0/0"
        ):
            return True
        return False

    def title(self, event):
        user = event.deep_get("username", default="<USER_NOT_FOUND>")
        group_id = event.deep_get("groupId", default="<GROUP_NOT_FOUND>")
        return f"MongoDB: [{user}] has allowed access to group [{group_id}] from anywhere"

    def alert_context(self, event):
        context = mongodb_alert_context(event)
        context["groupId"] = event.deep_get("groupId", default="<GROUP_NOT_FOUND>")
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

one_password_sensitive_item_tests: list[RuleTest] = [
    RuleTest(
        name="1Password - Sensitive Item Accessed",
        expected_result=True,
        log={
            "uuid": "ecd1d435c26440dc930ddfbbef201a11",
            "timestamp": "2022-02-23 20:27:17.071",
            "used_version": 2,
            "vault_uuid": "111111",
            "item_uuid": "ecd1d435c26440dc930ddfbbef201a11",
            "user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "2222222"},
            "client": {
                "app_name": "1Password Browser Extension",
                "app_version": "20195",
                "ip_address": "1.1.1.1.1",
                "os_name": "MacOSX",
                "os_version": "10.15.7",
                "platform_name": "Chrome",
                "platform_version": "4.0.4.102",
            },
            "p_log_type": "OnePassword.ItemUsage",
        },
    ),
    RuleTest(
        name="1Password - Regular Item Usage",
        expected_result=False,
        log={
            "uuid": "11111",
            "timestamp": "2022-02-23 20:27:17.071",
            "used_version": 2,
            "vault_uuid": "111111",
            "item_uuid": "1111111",
            "user": {"email": "homer@springfield.gov", "name": "Homer Simpson", "uuid": "2222222"},
            "client": {
                "app_name": "1Password Browser Extension",
                "app_version": "20195",
                "ip_address": "1.1.1.1.1",
                "os_name": "MacOSX",
                "os_version": "10.15.7",
                "platform_name": "Chrome",
                "platform_version": "4.0.4.102",
            },
            "p_log_type": "OnePassword.ItemUsage",
        },
    ),
]


@panther_managed
class OnePasswordSensitiveItem(Rule):
    id = "OnePassword.Sensitive.Item-prototype"
    dedup_period_minutes = 30
    display_name = "Configuration Required - Sensitive 1Password Item Accessed"
    enabled = False
    log_types = [LogType.ONEPASSWORD_ITEM_USAGE]
    default_reference = "https://support.1password.com/1password-com-items/"
    default_severity = Severity.LOW
    default_description = "Alerts when a user defined list of sensitive items in 1Password is accessed"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tags = ["Configuration Required", "1Password", "Credential Access:Unsecured Credentials"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    tests = one_password_sensitive_item_tests
    "\nThis rule detects access to high sensitivity items in your 1Password account. 1Password references\nthese items by their UUID so the SENSITIVE_ITEM_WATCHLIST below allows for the mapping of UUID to\nmeaningful name.\n\nThere is an alternative method for creating this rule that uses Panther's lookup table feature,\n(currently in beta). That rule can be found in the 1Password detection pack with the name\nBETA - Sensitive 1Password Item Accessed (onepassword_lut_sensitive_item_access.py)\n"
    SENSITIVE_ITEM_WATCHLIST = {"ecd1d435c26440dc930ddfbbef201a11": "demo_item"}

    def rule(self, event):
        return event.get("item_uuid") in self.SENSITIVE_ITEM_WATCHLIST.keys()

    def title(self, event):
        return f"A Sensitive 1Password Item was Accessed by user {deep_get(event, 'user', 'name')}"

    def alert_context(self, event):
        context = {
            "user": deep_get(event, "user", "name"),
            "item_name": deep_get(event, "p_enrichment", "1Password Translation", "item_uuid", "title"),
            "client": deep_get(event, "client", "app_name"),
            "ip_address": event.udm("source_ip"),
            "event_time": event.get("timestamp"),
        }
        return context



from typing import Any, Dict, Optional

UPLOAD_IN_PROGRESS_SUBSTR = "another upload"


def is_retryable_error(err: Optional[Dict[str, Any]]) -> bool:
    if err:
        if is_retryable_error_str(err.get("message", "")) or is_retryable_error_str(err.get("body", "")):
            return True
    return False


def is_retryable_error_str(err: str) -> bool:
    if not err:
        return False

    return (
        UPLOAD_IN_PROGRESS_SUBSTR in err
        or err == "upload failed"
        or "unknown error occurred" in err
        or "ddb lock" in err
        or "pload does not exist" in err
    )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsvpc_healthy_log_status_tests: list[RuleTest] = [
    RuleTest(name="Healthy Log Status", expected_result=False, log={"log-status": "OK", "p_log_type": "AWS.VPCFlow"}),
    RuleTest(
        name="Unhealthy Log Status",
        expected_result=True,
        log={"log-status": "SKIPDATA", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Healthy Log Status - OCSF",
        expected_result=False,
        log={"status_code": "OK", "p_log_type": "OCSF.NetworkActivity"},
    ),
    RuleTest(
        name="Unhealthy Log Status - OCSF",
        expected_result=True,
        log={"status_code": "SKIPDATA", "p_log_type": "OCSF.NetworkActivity"},
    ),
]


@panther_managed
class AWSVPCHealthyLogStatus(Rule):
    id = "AWS.VPC.HealthyLogStatus-prototype"
    display_name = "AWS VPC Healthy Log Status"
    log_types = [LogType.AWS_VPC_FLOW, LogType.OCSF_NETWORK_ACTIVITY]
    tags = ["AWS", "DataModel", "Security Control"]
    default_severity = Severity.LOW
    default_description = "Checks for the log status `SKIP-DATA`, which indicates that data was lost either to an internal server error or due to capacity constraints.\n"
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html#flow-log-records"
    default_runbook = "Determine if the cause of the issue is capacity constraints, and consider adjusting VPC Flow Log configurations accordingly.\n"
    tests = awsvpc_healthy_log_status_tests

    def rule(self, event):
        return event.udm("log_status") == "SKIPDATA"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardCloudflareFirewall(DataModel):
    id: str = "Standard.Cloudflare.Firewall"
    display_name: str = "Cloudflare Firewall"
    enabled: bool = True
    log_types: list[str] = [LogType.CLOUDFLARE_FIREWALL]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="source_ip", path="ClientIP"),
        DataModelMapping(name="user_agent", path="ClientRequestUserAgent"),
        DataModelMapping(name="http_status", path="EdgeResponseStatus"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcpgcsiam_changes_tests: list[RuleTest] = [
    RuleTest(
        name="GCS IAM Change",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {"principalEmail": "user@runpanther.io"},
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                    "requestAttributes": {"time": "2020-05-15T04:28:42.243082428Z", "auth": {}},
                    "destinationAttributes": {},
                },
                "serviceName": "storage.googleapis.com",
                "methodName": "storage.setIamPermissions",
                "authorizationInfo": [
                    {
                        "resource": "projects/_/buckets/jacks-test-bucket",
                        "permission": "storage.buckets.setIamPolicy",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                ],
                "resourceName": "projects/_/buckets/jacks-test-bucket",
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {"action": "ADD", "role": "roles/storage.objectViewer", "member": "allUsers"},
                        ],
                    },
                },
                "resourceLocation": {"currentLocations": ["us"]},
            },
            "insertId": "15cp9rve72xt1",
            "resource": {
                "type": "gcs_bucket",
                "labels": {"bucket_name": "jacks-test-bucket", "project_id": "western-verve-123456", "location": "us"},
            },
            "timestamp": "2020-05-15T04:28:42.237027213Z",
            "severity": "NOTICE",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "receiveTimestamp": "2020-05-15T04:28:42.900626148Z",
        },
    ),
]


@panther_managed
class GCPGCSIAMChanges(Rule):
    id = "GCP.GCS.IAMChanges-prototype"
    display_name = "GCP GCS IAM Permission Changes"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Google Cloud Storage", "Collection:Data From Cloud Storage Object"]
    reports = {"CIS": ["2.1"], "MITRE ATT&CK": ["TA0009:T1530"]}
    default_severity = Severity.LOW
    default_description = "Monitoring changes to Cloud Storage bucket permissions may reduce time to detect and correct permissions on sensitive Cloud Storage bucket and objects inside the bucket.\n"
    default_runbook = "Validate the GCS bucket change was safe."
    default_reference = "https://cloud.google.com/storage/docs/access-control/iam-permissions"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    tests = gcpgcsiam_changes_tests

    def rule(self, event):
        return (
            deep_get(event, "resource", "type") == "gcs_bucket"
            and deep_get(event, "protoPayload", "methodName") == "storage.setIamPermissions"
        )

    def dedup(self, event):
        return deep_get(event, "resource", "labels", "project_id", default="<UNKNOWN_PROJECT>")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_iocs import CRYPTO_MINING_DOMAINS

awsdns_crypto_domain_tests: list[RuleTest] = [
    RuleTest(
        name="Non Crypto Query",
        expected_result=False,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "dynamodb.us-west-2.amazonaws.com",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Non Crypto Query Trailing Period",
        expected_result=False,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "dynamodb.us-west-2.amazonaws.com.",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Crypto Query",
        expected_result=True,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "moneropool.ru",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Crypto Query Subdomain",
        expected_result=True,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "abc.abc.moneropool.ru",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Crypto Query Trailing Period",
        expected_result=True,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "moneropool.ru.",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Crypto Query Subdomain Trailing Period",
        expected_result=True,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "abc.abc.moneropool.ru.",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Checking Against Subdomain IOC",
        expected_result=True,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "webservicepag.webhop.net",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Checking Against Subdomain IOC Trailing Period",
        expected_result=True,
        log={
            "account_id": "0123456789",
            "answers": {"Class": "IN", "Rdata": "1.2.3.4", "Type": "A"},
            "query_class": "IN",
            "query_name": "webservicepag.webhop.net.",
            "query_timestamp": "2022-06-25 00:27:53",
            "query_type": "A",
            "rcode": "NOERROR",
            "region": "us-west-2",
            "srcaddr": "5.6.7.8",
            "srcids": {"instance": "i-0abc234"},
            "srcport": "8888",
            "transport": "UDP",
            "version": "1.100000",
            "vpc_id": "vpc-abc123",
            "p_log_type": "AWS.VPCDns",
        },
    ),
    RuleTest(
        name="Non Crypto Query Trailing Period - OCSF",
        expected_result=False,
        log={
            "activity_id": 2,
            "activity_name": "Response",
            "answers": [{"class": "IN", "rdata": "1.2.3.4", "type": "AAAA"}],
            "category_name": "Network Activity",
            "category_uid": 4,
            "class_name": "DNS Activity",
            "class_uid": 4003,
            "cloud": {"provider": "AWS", "region": "us-west-2"},
            "connection_info": {"direction": "Unknown", "direction_id": 0, "protocol_name": "UDP"},
            "disposition": "Unknown",
            "disposition_id": 0,
            "metadata": {
                "product": {
                    "feature": {"name": "Resolver Query Logs"},
                    "name": "Route 53",
                    "vendor_name": "AWS",
                    "version": "1.100000",
                },
                "profiles": ["cloud", "security_control"],
                "version": "1.100000",
            },
            "query": {"class": "IN", "hostname": "dynamodb.us-west-2.amazonaws.com.", "type": "AAAA"},
            "rcode": "NoError",
            "rcode_id": 0,
            "severity": "Informational",
            "severity_id": 1,
            "src_endpoint": {"instance_uid": "i-0abc234", "ip": "5.6.7.8", "port": "8888", "vpc_uid": "vpc-abc123"},
            "time": "2022-06-25 00:27:53",
            "type_name": "DNS Activity: Response",
            "type_uid": 400302,
            "p_log_type": "OCSF.DnsActivity",
        },
    ),
    RuleTest(
        name="Crypto Query - OCSF",
        expected_result=True,
        log={
            "activity_id": 2,
            "activity_name": "Response",
            "answers": [{"class": "IN", "rdata": "1.2.3.4", "type": "AAAA"}],
            "category_name": "Network Activity",
            "category_uid": 4,
            "class_name": "DNS Activity",
            "class_uid": 4003,
            "cloud": {"provider": "AWS", "region": "us-west-2"},
            "connection_info": {"direction": "Unknown", "direction_id": 0, "protocol_name": "UDP"},
            "disposition": "Unknown",
            "disposition_id": 0,
            "metadata": {
                "product": {
                    "feature": {"name": "Resolver Query Logs"},
                    "name": "Route 53",
                    "vendor_name": "AWS",
                    "version": "1.100000",
                },
                "profiles": ["cloud", "security_control"],
                "version": "1.100000",
            },
            "query": {"class": "IN", "hostname": "moneropool.ru", "type": "AAAA"},
            "rcode": "NoError",
            "rcode_id": 0,
            "severity": "Informational",
            "severity_id": 1,
            "src_endpoint": {"instance_uid": "i-0abc234", "ip": "5.6.7.8", "port": "8888", "vpc_uid": "vpc-abc123"},
            "time": "2022-06-25 00:27:53",
            "type_name": "DNS Activity: Response",
            "type_uid": 400302,
            "p_log_type": "OCSF.DnsActivity",
        },
    ),
]


@panther_managed
class AWSDNSCryptoDomain(Rule):
    default_description = (
        "Identifies clients that may be performing DNS lookups associated with common currency mining pools."
    )
    display_name = "AWS DNS Crypto Domain"
    reports = {"MITRE ATT&CK": ["TA0040:T1496"]}
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html"
    default_severity = Severity.HIGH
    log_types = [LogType.AWS_VPC_DNS, LogType.OCSF_DNS_ACTIVITY]
    id = "AWS.DNS.Crypto.Domain-prototype"
    tests = awsdns_crypto_domain_tests

    def rule(self, event):
        query_name = event.udm("dns_query")
        if not query_name:
            return False
        for domain in CRYPTO_MINING_DOMAINS:
            if query_name.rstrip(".").endswith(domain):
                return True
        return False

    def title(self, event):
        return f"[{event.udm('source_ip')}:{event.udm('source_port')}] made a DNS query for crypto mining domain: [{event.udm('dns_query')}]."

    def dedup(self, event):
        return f"{event.udm('source_ip')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

decoy_s3_accessed_tests: list[RuleTest] = [
    RuleTest(
        name="S3-Decoy-Accessed",
        expected_result=True,
        log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "GetObject",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "RemoteIpDetails": {
                        "City": {},
                        "Country": {},
                        "GeoLocation": {},
                        "IpAddressV4": "111.111.111.111",
                        "Organization": {},
                    },
                    "ServiceName": "s3.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 00:26:57.000000000",
            "Description": "Private decoy S3 bucket panther-databucket was accessed by arn:aws:iam::123456789012:user/tester. This S3 bucket has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {"Severity": {"Label": "HIGH", "Normalized": 70}, "Types": ["Unusual Behaviors"]},
            "GeneratorId": "s3.amazonaws.com",
            "Id": "ABC9ONWNS3155VIEJC8U",
            "ProcessedAt": "2024-05-24T00:27:12.237Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd234-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "[Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36]",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:service:region:123456789012:resource/12345ab6-436d-4d59-ac58-ed6b3127e440",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "arn:aws:s3:::panther-databucket",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Tags": {
                        "aws:cloudformation:logical-id": "DataBucket",
                        "aws:cloudformation:stack-id": "arn:aws:cloudformation:us-east-1:123456789012:stack/Panther/a1b2c345-12f6-11ef-8c74-12deb08d9ef1",
                        "aws:cloudformation:stack-name": "Panther",
                    },
                    "Type": "AwsS3Bucket",
                },
                {
                    "Id": "arn:aws:s3:::panther-databucket/object",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsS3Object",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABCDEFG1HIJ2KLMNOPQR",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABCDEFG1HIJ2KLMNOPQR:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {"CreationDate": "2024-05-23T20:20:57Z", "MfaAuthenticated": False},
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:role/tester",
                                    "PrincipalId": "ABCDEFG1HIJ2KLMNOPQR",
                                    "Type": "Role",
                                    "UserName": "tester",
                                },
                            },
                        },
                    },
                    "Id": "ABCDEFG1HIJ2KLMNOPQR",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "ABCDEFG1HIJ2KLMNOPQR",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Suspicious activity detected accessing private decoy S3 bucket panther-databucket",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 00:26:57.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_ip_addresses": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 00:26:57.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 00:30:04.569556803",
            "p_row_id": "624c79c882affe88a1dce9c31fb68f0e",
            "p_schema_version": 0,
            "p_source_id": "e29fd64f-53d9-43ab-92ca-575a8af289e6",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
    RuleTest(
        name="S3-Decoy-Not-Accessed",
        expected_result=False,
        log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "ListBuckets",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "RemoteIpDetails": {
                        "City": {},
                        "Country": {},
                        "GeoLocation": {},
                        "IpAddressV4": "111.111.111.111",
                        "Organization": {},
                    },
                    "ServiceName": "s3.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 00:26:57.000000000",
            "Description": "Private decoy S3 bucket panther-databucket was not accessed by arn:aws:iam::123456789012:user/tester. This S3 bucket has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {"Severity": {"Label": "HIGH", "Normalized": 70}, "Types": ["Unusual Behaviors"]},
            "GeneratorId": "s3.amazonaws.com",
            "Id": "ABC9ONWNS3155VIEJC8U",
            "ProcessedAt": "2024-05-24T00:27:12.237Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd234-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "[Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/124.0.0.0 Safari/537.36]",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:service:region:123456789012:resource/12345ab6-436d-4d59-ac58-ed6b3127e440",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "arn:aws:s3:::panther-databucket",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Tags": {
                        "aws:cloudformation:logical-id": "DataBucket",
                        "aws:cloudformation:stack-id": "arn:aws:cloudformation:us-east-1:123456789012:stack/Panther/a1b2c345-12f6-11ef-8c74-12deb08d9ef1",
                        "aws:cloudformation:stack-name": "Panther",
                    },
                    "Type": "AwsS3Bucket",
                },
                {
                    "Id": "arn:aws:s3:::panther-databucket/object",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsS3Object",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABCDEFG1HIJ2KLMNOPQR",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABCDEFG1HIJ2KLMNOPQR:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {"CreationDate": "2024-05-23T20:20:57Z", "MfaAuthenticated": False},
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:role/tester",
                                    "PrincipalId": "ABCDEFG1HIJ2KLMNOPQR",
                                    "Type": "Role",
                                    "UserName": "tester",
                                },
                            },
                        },
                    },
                    "Id": "ABCDEFG1HIJ2KLMNOPQR",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "ABCDEFG1HIJ2KLMNOPQR",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Non-Suspicious activity detected accessing private decoy S3 bucket panther-databucket",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 00:26:57.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_ip_addresses": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 00:26:57.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 00:30:04.569556803",
            "p_row_id": "624c79c882affe88a1dce9c31fb68f0e",
            "p_schema_version": 0,
            "p_source_id": "e29fd64f-53d9-43ab-92ca-575a8af289e6",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
]


@panther_managed
class DecoyS3Accessed(Rule):
    id = "Decoy.S3.Accessed-prototype"
    display_name = "Decoy S3 Accessed"
    enabled = False
    log_types = [LogType.AWS_SECURITY_FINDING_FORMAT]
    default_severity = Severity.HIGH
    default_description = "Actor accessed S3 Manager decoy secret"
    default_reference = "https://aws.amazon.com/blogs/security/how-to-detect-suspicious-activity-in-your-aws-account-by-using-private-decoy-resources/"
    inline_filters = [{"All": []}]
    tests = decoy_s3_accessed_tests

    def rule(self, event):
        # List of suspicious API events
        # NOTE: There may be more API events that's not listed
        suspicious_api_events = [
            "HeadObject",
            "GetObject",
            "GetObjectAcl",
            "GetObjectAttributes",
            "GetObjectLegalHold",
            "GetObjectLockConfiguration",
            "GetObjectRetention",
            "GetObjectTagging",
            "GetObjectTorrent",
            "PutObject",
            "PutObjectAcl",
            "PutObjectLegalHold",
            "PutObjectLockConfiguration",
            "PutObjectRetention",
            "PutObjectTagging",
            "SelectObjectContent",
            "DeleteObject",
            "DeleteObjects",
            "DeleteObjectTagging",
        ]
        # Return True if the API value is in the list of suspicious API events
        if event["GeneratorId"] == "s3.amazonaws.com":
            # Extract the API value from the event
            api_value = event["Action"]["AwsApiCallAction"]["Api"]
            return api_value in suspicious_api_events
        return False

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method will act as dedup string.
        # NOTE: Not sure if the offending actor Id will always be in the 0th index of Resources
        # It's possible to just return the Title as a whole string
        secret = event["Resources"][0]["Id"]
        return f"Suspicious activity detected accessing private decoy S3 bucket {secret}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

sentinel_one_threats_tests: list[RuleTest] = [
    RuleTest(
        name="malicious event",
        expected_result=True,
        log={
            "accountid": "123456789",
            "accountname": "Account1",
            "activitytype": 19,
            "activityuuid": "123-456-678-89",
            "agentid": "1111112222233333",
            "createdat": "2022-12-07 16:08:55.703",
            "data": {
                "accountname": "Account1",
                "computername": "BobsPC",
                "confidencelevel": "malicious",
                "filecontenthash": "cf8bd9dfddff007f75adf4c2be48005cea317c62",
                "filedisplayname": "eicar.txt",
                "filepath": "/home/ubuntu/eicar.txt",
                "fullscopedetails": "Group Testing in Site Default site of Account Account1",
                "fullscopedetailspath": "Global / Account1 / Default site / Testing",
                "groupname": "Testing",
                "sitename": "Default site",
                "threatclassification": "Virus",
                "threatclassificationsource": "Cloud",
            },
            "groupid": "12345",
            "groupname": "Testing",
            "id": "11111111",
            "primarydescription": "Threat with confidence level malicious detected: eicar.txt",
            "secondarydescription": "cf8bd9dfddff007f75adf4c2be48005cea317c62",
            "siteid": "456789",
            "sitename": "Default site",
            "threatid": "123456789",
            "updatedat": "2022-12-07 16:08:55.698",
        },
    ),
    RuleTest(
        name="non-threat event",
        expected_result=False,
        log={
            "accountid": "12345",
            "accountname": "Account1",
            "activitytype": 90,
            "activityuuid": "123-456-789",
            "agentid": "111111",
            "createdat": "2022-12-07 16:06:35.483",
            "data": {
                "accountname": "Account1",
                "computername": "BobsPC",
                "createdat": "2022-12-07T16:06:35.477827Z",
                "fullscopedetails": "Group Testing in Site Default site of Account Account1",
                "fullscopedetailspath": "Global / Account1 / Default site / Testing",
                "groupname": "Testing",
                "scopelevel": "Group",
                "scopename": "Testing",
                "sitename": "Default site",
                "status": "started",
            },
            "groupid": "11234",
            "groupname": "Testing",
            "id": "123564",
            "primarydescription": "Agent BobsPC started full disk scan at Wed, 07 Dec 2022, 16:06:35 UTC.",
            "siteid": "12345",
            "sitename": "Default site",
            "updatedat": "2022-12-07 16:06:35.479",
        },
    ),
    RuleTest(
        name="suspicious event",
        expected_result=True,
        log={
            "accountid": "123456789",
            "accountname": "Account1",
            "activitytype": 19,
            "activityuuid": "123-456-678-89",
            "agentid": "1111112222233333",
            "createdat": "2022-12-07 16:08:55.703",
            "data": {
                "accountname": "Account1",
                "computername": "BobsPC",
                "confidencelevel": "suspicious",
                "filecontenthash": "cf8bd9dfddff007f75adf4c2be48005cea317c62",
                "filedisplayname": "eicar.txt",
                "filepath": "/home/ubuntu/eicar.txt",
                "fullscopedetails": "Group Testing in Site Default site of Account Account1",
                "fullscopedetailspath": "Global / Account1 / Default site / Testing",
                "groupname": "Testing",
                "sitename": "Default site",
                "threatclassification": "Virus",
                "threatclassificationsource": "Cloud",
            },
            "groupid": "12345",
            "groupname": "Testing",
            "id": "11111111",
            "primarydescription": "Threat with confidence level malicious detected: eicar.txt",
            "secondarydescription": "cf8bd9dfddff007f75adf4c2be48005cea317c62",
            "siteid": "456789",
            "sitename": "Default site",
            "threatid": "123456789",
            "updatedat": "2022-12-07 16:08:55.698",
        },
    ),
]


@panther_managed
class SentinelOneThreats(Rule):
    default_description = "Passthrough SentinelOne Threats "
    display_name = "SentinelOne Threats"
    default_reference = "https://www.sentinelone.com/blog/feature-spotlight-introducing-the-new-threat-center/"
    default_severity = Severity.HIGH
    log_types = [LogType.SENTINELONE_ACTIVITY]
    id = "SentinelOne.Threats-prototype"
    tests = sentinel_one_threats_tests  # New Malicious Threat Not Mitigated
    # New Malicious Threat Not Mitigated
    # New Suspicious Threat Not Mitigated
    # New Suspicious Threat Not Mitigated
    NEW_THREAT_ACTIVITYTYPES = [19, 4108, 4003, 4109]

    def rule(self, event):
        return event.get("activitytype") in self.NEW_THREAT_ACTIVITYTYPES

    def title(self, event):
        return f"SentinelOne - [{deep_get(event, 'data', 'confidencelevel', default='')}] level [{deep_get(event, 'data', 'threatclassification', default='')}] threat detected from [{deep_get(event, 'data', 'threatclassificationsource', default='')}]."

    def dedup(self, event):
        return f"s1threat:{event.get('id', '')}"

    def severity(self, event):
        if deep_get(event, "data", "confidencelevel", default="") == "malicious":
            return "CRITICAL"
        return "HIGH"

    def alert_context(self, event):
        return {
            "primarydescription": event.get("primarydescription", ""),
            "accountname": event.get("accountname", ""),
            "accountid": event.get("accountid", ""),
            "siteid": event.get("siteid", ""),
            "sitename": event.get("sitename", ""),
            "threatid": event.get("threatid", ""),
            "groupid": event.get("groupid", ""),
            "groupname": event.get("groupname", ""),
            "activityuuid": event.get("activityuuid", ""),
            "agentid": event.get("agentid", ""),
            "id": event.get("id", ""),
            "data": event.get("data", {}),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.helpers.panther_default import aws_cloudtrail_success

awsec2_vpc_modified_tests: list[RuleTest] = [
    RuleTest(
        name="VPC Modified",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "CreateVpc",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.ec2.amazonaws.com",
            "requestParameters": {
                "cidrBlock": "0.0.0.0/26",
                "instanceTenancy": "default",
                "amazonProvidedIpv6CidrBlock": False,
            },
            "responseElements": {
                "requestID": "1",
                "vpc": {
                    "vpcId": "vpc-1",
                    "state": "pending",
                    "ownerId": "123456789012",
                    "cidrBlock": "0.0.0.0/26",
                    "cidrBlockAssociationSet": {
                        "items": [
                            {
                                "cidrBlock": "0.0.0.0/26",
                                "associationId": "vpc-cidr-assoc-1",
                                "cidrBlockState": {"state": "associated"},
                            },
                        ],
                    },
                    "ipv6CidrBlockAssociationSet": {},
                    "dhcpOptionsId": "dopt-1",
                    "instanceTenancy": "default",
                    "tagSet": {},
                    "isDefault": False,
                },
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="VPC Not Modified",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "false", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "DescribeVpcs",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {"vpcSet": {}, "filterSet": {}},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Error Modifying VPC",
        expected_result=False,
        log={
            "errorCode": "UnknownParameter",
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "ec2.amazonaws.com",
            "eventName": "CreateVpc",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.ec2.amazonaws.com",
            "requestParameters": {
                "cidrBlock": "0.0.0.0/26",
                "instanceTenancy": "default",
                "amazonProvidedIpv6CidrBlock": False,
            },
            "responseElements": {
                "requestID": "1",
                "vpc": {
                    "vpcId": "vpc-1",
                    "state": "pending",
                    "ownerId": "123456789012",
                    "cidrBlock": "0.0.0.0/26",
                    "cidrBlockAssociationSet": {
                        "items": [
                            {
                                "cidrBlock": "0.0.0.0/26",
                                "associationId": "vpc-cidr-assoc-1",
                                "cidrBlockState": {"state": "associated"},
                            },
                        ],
                    },
                    "ipv6CidrBlockAssociationSet": {},
                    "dhcpOptionsId": "dopt-1",
                    "instanceTenancy": "default",
                    "tagSet": {},
                    "isDefault": False,
                },
            },
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSEC2VPCModified(Rule):
    id = "AWS.EC2.VPCModified-prototype"
    display_name = "EC2 VPC Modified"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["3.14"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.INFO
    dedup_period_minutes = 720
    default_description = "An EC2 VPC was modified."
    default_runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-ec2-vpc-modified"
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/configure-your-vpc.html"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = awsec2_vpc_modified_tests
    # API calls that are indicative of an EC2 VPC modification
    EC2_VPC_MODIFIED_EVENTS = {
        "CreateVpc",
        "DeleteVpc",
        "ModifyVpcAttribute",
        "AcceptVpcPeeringConnection",
        "CreateVpcPeeringConnection",
        "DeleteVpcPeeringConnection",
        "RejectVpcPeeringConnection",
        "AttachClassicLinkVpc",
        "DetachClassicLinkVpc",
        "DisableVpcClassicLink",
        "EnableVpcClassicLink",
    }

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.EC2_VPC_MODIFIED_EVENTS

    def dedup(self, event):
        return event.get("recipientAccountId")

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, pattern_match

aws_macie_evasion_tests: list[RuleTest] = [
    RuleTest(
        name="ListMembers",
        expected_result=False,
        log={
            "awsRegion": "us-west-1",
            "eventCategory": "Management",
            "eventID": "5b3e4cf6-c37d-4c8c-9016-b8444a37ceaa",
            "eventName": "ListMembers",
            "eventSource": "macie2.amazonaws.com",
            "eventTime": "2022-09-27 18:11:33",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789012"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789012:role/Admin",
                "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
            ],
            "p_any_ip_addresses": ["178.253.78.209"],
            "p_any_trace_ids": ["AAAASSSST64ZTHFY7777"],
            "p_event_time": "2022-09-27 18:11:33",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-27 18:16:43.428",
            "p_row_id": "665d45a409cad7d68ff7bbd4138d02",
            "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
            "p_source_label": "CloudTrail Test",
            "readOnly": True,
            "recipientAccountId": "123456789012",
            "requestID": "2164bbea-3eb0-444b-8e10-8ba53b3460b6",
            "requestParameters": {"maxResults": "1", "onlyAssociated": "true"},
            "sourceIPAddress": "178.253.78.209",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "AAAASSSST64ZTHFY7777",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                "principalId": "AAAAA44444LE6DYFKKKKK:Jack",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-27T17:56:01Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/Admin",
                        "principalId": "AAAAA44444LE6DYFKKKKK",
                        "type": "Role",
                        "userName": "Admin",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="UpdateSession",
        expected_result=True,
        log={
            "awsRegion": "us-east-2",
            "eventCategory": "Management",
            "eventID": "63033dfd-08c9-42f3-80ae-dca45e86ae84",
            "eventName": "UpdateMacieSession",
            "eventSource": "macie2.amazonaws.com",
            "eventTime": "2022-09-27 19:59:08",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789012"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789012:role/Admin",
                "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
            ],
            "p_any_ip_addresses": ["46.91.25.204"],
            "p_any_trace_ids": ["ASIASWJRT64Z42HFV6QX"],
            "p_event_time": "2022-09-27 19:59:08",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-27 20:02:43.816",
            "p_row_id": "665d45a409cad7d68ff7bbd4138123",
            "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
            "p_source_label": "CloudTrail Test",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1b9981dc-21d2-4f77-92b0-69e23c8a40de",
            "requestParameters": {"findingPublishingFrequency": "SIX_HOURS"},
            "sourceIPAddress": "46.91.25.204",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "ASIASWJRT64Z42HFV6QX",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                "principalId": "AAAAA44444LE6DYFKKKKK:Jack",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-27T17:56:01Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/Admin",
                        "principalId": "AAAAA44444LE6DYFKKKKK",
                        "type": "Role",
                        "userName": "Admin",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="UpdateSession (Macie v1 event)",
        expected_result=True,
        log={
            "awsRegion": "us-east-2",
            "eventCategory": "Management",
            "eventID": "63033dfd-08c9-42f3-80ae-dca45e86ae84",
            "eventName": "UpdateMacieSession",
            "eventSource": "macie.amazonaws.com",
            "eventTime": "2022-09-27 19:59:08",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789012"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789012:role/Admin",
                "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
            ],
            "p_any_ip_addresses": ["46.91.25.204"],
            "p_any_trace_ids": ["ASIASWJRT64Z42HFV6QX"],
            "p_event_time": "2022-09-27 19:59:08",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-27 20:02:43.816",
            "p_row_id": "665d45a409cad7d68ff7bbd4138123",
            "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
            "p_source_label": "CloudTrail Test",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1b9981dc-21d2-4f77-92b0-69e23c8a40de",
            "requestParameters": {"findingPublishingFrequency": "SIX_HOURS"},
            "sourceIPAddress": "46.91.25.204",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "ASIASWJRT64Z42HFV6QX",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                "principalId": "AAAAA44444LE6DYFKKKKK:Jack",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-27T17:56:01Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/Admin",
                        "principalId": "AAAAA44444LE6DYFKKKKK",
                        "type": "Role",
                        "userName": "Admin",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSMacieEvasion(Rule):
    id = "AWS.Macie.Evasion-prototype"
    display_name = "AWS Macie Disabled/Updated"
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.MEDIUM
    default_description = "Amazon Macie is a data security and data privacy service to discover and protect sensitive data. Security teams use Macie to detect open S3 Buckets that could have potentially sensitive data in it along with policy violations, such as missing Encryption. If an attacker disables Macie, it could potentially hide data exfiltration.\n"
    default_reference = "https://aws.amazon.com/macie/"
    default_runbook = "Analyze the events to ensure it's not normal maintenance. If it's abnormal, run the Indicator Search on the UserIdentity:Arn for the past hour and analyze other services accessed/changed.\n"
    threshold = 5
    summary_attributes = [
        "awsRegion",
        "eventName",
        "p_any_aws_arns",
        "p_any_ip_addresses",
        "userIdentity:type",
        "userIdentity:arn",
    ]
    tests = aws_macie_evasion_tests
    MACIE_EVENTS = {
        "ArchiveFindings",
        "CreateFindingsFilter",
        "DeleteMember",
        "DisassociateFromMasterAccount",
        "DisassociateMember",
        "DisableMacie",
        "DisableOrganizationAdminAccount",
        "UpdateFindingsFilter",
        "UpdateMacieSession",
        "UpdateMemberSession",
        "UpdateClassificationJob",
    }

    def rule(self, event):
        return event.get("eventName") in self.MACIE_EVENTS and pattern_match(
            event.get("eventSource"),
            "macie*.amazonaws.com",
        )

    def title(self, event):
        account = event.get("recipientAccountId")
        user_arn = deep_get(event, "userIdentity", "arn")
        return f"AWS Macie in AWS Account [{account}] Disabled/Updated by [{user_arn}]"



from pypanther.base import DataModel
from pypanther.get import get_panther_data_models


class DataModelCache:
    def __init__(self):
        self.data_models = {}

        for data_model in get_panther_data_models():
            for log_type in data_model.log_types:
                self.data_models[log_type] = data_model()

    def data_model_of_logtype(self, log_type: str) -> DataModel:
        return self.data_models.get(log_type)


DATA_MODEL_CACHE = DataModelCache()



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_team_modified_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Team Deleted",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "team.destroy",
            "created_at": 1621305118553,
            "data": {"team": "my-org/my-team"},
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Team Created",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "team.create",
            "created_at": 1621305118553,
            "data": {"team": "my-org/my-team"},
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Team Add repository",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "team.add_repository",
            "created_at": 1621305118553,
            "data": {"team": "my-org/my-team"},
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GitHubTeamModified(Rule):
    id = "GitHub.Team.Modified-prototype"
    display_name = "GitHub Team Modified"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Initial Access:Supply Chain Compromise"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195"]}
    default_reference = "https://docs.github.com/en/organizations/organizing-members-into-teams"
    default_severity = Severity.INFO
    default_description = "Detects when a team is modified in some way, such as adding a new team, deleting a team, modifying members, or a change in repository control."
    tests = git_hub_team_modified_tests

    def rule(self, event):
        if not event.get("action").startswith("team"):
            return False
        return (
            event.get("action") == "team.add_member"
            or event.get("action") == "team.add_repository"
            or event.get("action") == "team.change_parent_team"
            or (event.get("action") == "team.create")
            or (event.get("action") == "team.destroy")
            or (event.get("action") == "team.remove_member")
            or (event.get("action") == "team.remove_repository")
        )

    def title(self, event):
        action_mappings = {
            "create": "created team",
            "destroy": "deleted team",
            "add_member": f"added member [{event.get('user')}] to team",
            "remove_member": f"removed member [{event.get('user')}] from team",
            "add_repository": f"added repository [{event.get('repo')}] to team",
            "removed_repository": f"removed repository [{event.get('repo')}] from team",
            "change_parent_team": "changed parent team for team",
        }
        action_key = event.get("action").split(".")[1]
        action = action_mappings.get(action_key, event.get("action"))
        team_name = event.get("team") if "team" in event else "<MISSING_TEAM>"
        return f"GitHub.Audit: User [{event.udm('actor_user')}] {action} [{team_name}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

box_large_number_downloads_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="User Download",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example", "name": "Bob Cat"},
            "event_type": "DOWNLOAD",
            "source": {"id": "12345678", "type": "user", "login": "user@example", "name": "Bob Cat"},
        },
    ),
]


@panther_managed
class BoxLargeNumberDownloads(Rule):
    id = "Box.Large.Number.Downloads-prototype"
    display_name = "Box Large Number of Downloads"
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Exfiltration:Exfiltration Over Web Service"]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.LOW
    default_description = "A user has exceeded the threshold for number of downloads within a single time frame.\n"
    default_reference = "https://support.box.com/hc/en-us/articles/360043697134-Download-Files-and-Folders-from-Box"
    default_runbook = "Investigate whether this user's download activity is expected.  Investigate the cause of this download activity.\n"
    summary_attributes = ["ip_address"]
    threshold = 100
    tests = box_large_number_downloads_tests

    def rule(self, event):
        return event.get("event_type") == "DOWNLOAD"

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'login', default='<UNKNOWN_USER>')}] exceeded threshold for number of downloads in the configured time frame."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_google_access_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={"id": {"applicationName": "login"}, "type": "login"},
    ),
    RuleTest(
        name="Resource Accessed by Google",
        expected_result=True,
        log={"id": {"applicationName": "access_transparency"}, "type": "GSUITE_RESOURCE"},
    ),
]


@panther_managed
class GSuiteGoogleAccess(Rule):
    id = "GSuite.GoogleAccess-prototype"
    display_name = "Google Accessed a GSuite Resource"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.LOW
    default_description = (
        "Google accessed one of your GSuite resources directly, most likely in response to a support incident.\n"
    )
    default_reference = "https://support.google.com/a/answer/9230474?hl=en"
    default_runbook = "Your GSuite Super Admin can visit the Access Transparency report in the GSuite Admin Dashboard to see more details about the access.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_google_access_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "access_transparency":
            return False
        return bool(event.get("type") == "GSUITE_RESOURCE")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context

notion_teamspace_owner_added_tests: list[RuleTest] = [
    RuleTest(
        name="Member Added",
        expected_result=False,
        log={
            "event": {
                "actor": {
                    "id": "c16137bb-5078-4eac-b026-5cbd2f9a027a",
                    "object": "user",
                    "person": {"email": "bill@example.com"},
                    "type": "person",
                },
                "details": {
                    "member": {
                        "id": "c16137bb-5078-4eac-b026-5cbd2f9a027a",
                        "object": "user",
                        "person": {"email": "bob@example.com"},
                        "type": "person",
                    },
                    "role": "member",
                    "target": {"id": "b8db234d-71eb-49e2-a5ed-7935ca764920", "name": "General", "object": "teamspace"},
                },
                "id": "eed75a56-ca1b-453b-afd8-73789bc19398",
                "ip_address": "11.22.33.44",
                "platform": "web",
                "timestamp": "2023-12-13 16:20:14.966000000",
                "type": "teamspace.permissions.member_added",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            },
        },
    ),
    RuleTest(
        name="Owner Added",
        expected_result=True,
        log={
            "event": {
                "actor": {
                    "id": "c16137bb-5078-4eac-b026-5cbd2f9a027a",
                    "object": "user",
                    "person": {"email": "malicious.insider@example.com"},
                    "type": "person",
                },
                "details": {
                    "member": {
                        "id": "c16137bb-5078-4eac-b026-5cbd2f9a027a",
                        "object": "user",
                        "person": {"email": "bad.dude@example.com"},
                        "type": "person",
                    },
                    "new_role": "owner",
                    "target": {"id": "b8db234d-71eb-49e2-a5ed-7935ca764920", "name": "General", "object": "teamspace"},
                },
                "id": "6019b995-0158-4430-8263-89ad7905bd1d",
                "ip_address": "11.22.33.44",
                "platform": "web",
                "timestamp": "2023-12-13 16:38:04.264000000",
                "type": "teamspace.permissions.member_role_updated",
                "workspace_id": "ea65b016-6abc-4dcf-808b-e119617b55d1",
            },
        },
    ),
]


@panther_managed
class NotionTeamspaceOwnerAdded(Rule):
    id = "Notion.TeamspaceOwnerAdded-prototype"
    display_name = "Notion Teamspace Owner Added"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Privilege Escalation"]
    default_description = "A Notion User was added as a Teamspace owner."
    default_severity = Severity.MEDIUM
    default_runbook = "Possible Privilege Escalation. Follow up with the Notion User to determine if this was done for a valid business reason."
    tests = notion_teamspace_owner_added_tests

    def rule(self, event):
        added = (
            event.deep_get("event", "type", default="") == "teamspace.permissions.member_added"
            and event.deep_get("event", "details", "role", default="") == "owner"
        )
        updated = (
            event.deep_get("event", "type", default="") == "teamspace.permissions.member_role_updated"
            and event.deep_get("event", "details", "new_role", default="") == "owner"
        )
        return added or updated

    def title(self, event):
        actor = event.deep_get("event", "actor", "person", "email", default="NO_ACTOR_FOUND")
        member = event.deep_get("event", "details", "member", "person", "email", default="NO_MEMBER_FOUND")
        teamspace = event.deep_get("event", "details", "target", "name", default="NO_TEAMSPACE_FOUND")
        return f"[{actor}] added [{member}] as owner of [{teamspace}] Teamspace"

    def alert_context(self, event):
        return notion_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_org_ip_allowlist_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - IP Allow list modified",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "ip_allow_list_entry.create",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "org": "my-org",
        },
    ),
    RuleTest(
        name="GitHub - IP Allow list disabled",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "ip_allow_list.disable",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
        },
    ),
    RuleTest(
        name="GitHub - Non IP Allow list action",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "org.invite_user",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
        },
    ),
]


@panther_managed
class GitHubOrgIpAllowlist(Rule):
    id = "GitHub.Org.IpAllowlist-prototype"
    display_name = "GitHub Org IP Allow List modified"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Persistence:Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_severity = Severity.MEDIUM
    summary_attributes = ["actor", "action"]
    default_description = "Detects changes to a GitHub Org IP Allow List"
    default_runbook = "Verify that the change was authorized and appropriate."
    default_reference = (
        "https://docs.github.com/en/apps/maintaining-github-apps/managing-allowed-ip-addresses-for-a-github-app"
    )
    tests = git_hub_org_ip_allowlist_tests
    ALLOWLIST_ACTIONS = [
        "ip_allow_list.enable",
        "ip_allow_list.disable",
        "ip_allow_list.enable_for_installed_apps",
        "ip_allow_list.disable_for_installed_apps",
        "ip_allow_list_entry.create",
        "ip_allow_list_entry.update",
        "ip_allow_list_entry.destroy",
    ]

    def rule(self, event):
        return event.get("action").startswith("ip_allow_list") and event.get("action") in self.ALLOWLIST_ACTIONS

    def title(self, event):
        return f"GitHub Org IP Allow list modified by {event.get('actor')}."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

aws_cloud_trail_account_discovery_tests: list[RuleTest] = [
    RuleTest(
        name="DescribeAccount",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "0b51d284-19f7-42cf-a103-276602aeada5",
            "eventName": "DescribeAccount",
            "eventSource": "organizations.amazonaws.com",
            "eventTime": "2022-11-21 18:06:52",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789123:role/TestUser",
                "arn:aws:sts::123456789123:assumed-role/TestUser/test_123456789123",
            ],
            "p_any_ip_addresses": ["1.1.1.1"],
            "p_any_trace_ids": ["ASIA3JHVJH35KB7LJHV2"],
            "p_any_usernames": ["TestUser"],
            "p_event_time": "2022-11-21 18:06:52",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-21 18:07:38.9",
            "p_row_id": "824956f0377f98908684d8de14d3d612",
            "p_source_id": "5f9f0f60-9c56-4027-b93a-8bab3019f0f1",
            "p_source_label": "Cloudtrail",
            "readOnly": True,
            "recipientAccountId": "123456789123",
            "requestID": "1c40241b-c59c-4d4a-8301-b612545f9c5c",
            "requestParameters": {"accountId": "123456789123"},
            "sourceIPAddress": "1.1.1.1",
            "tlsDetails": {
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "organizations.us-east-1.amazonaws.com",
                "tlsVersion": "TLSv1.2",
            },
            "userAgent": "Boto3/1.26.2 Python/3.10.8 Linux/4.14.294-220.533.amzn2.x86_64 exec-env/AWS_ECS_FARGATE Botocore/1.29.2",
            "userIdentity": {
                "accessKeyId": "ASIA3JHVJH35KB7LJHV2",
                "accountId": "123456789123",
                "arn": "arn:aws:sts::123456789123:assumed-role/TestUser/test_123456789123",
                "principalId": "AR0A354LKJXC87G9XC89V:test_123456789123",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-11-21T18:06:36Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123456789123",
                        "arn": "arn:aws:iam::123456789123:role/TestUser",
                        "principalId": "AR0A354LKJXC87G9XC89V",
                        "type": "Role",
                        "userName": "TestUser",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="GetAlternateContact",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "cd05c51d-fee2-4003-b9c5-385f28ad5b29",
            "eventName": "GetAlternateContact",
            "eventSource": "billingconsole.amazonaws.com",
            "eventTime": "2022-11-23 21:06:45",
            "eventType": "AwsConsoleAction",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789123"],
            "p_any_aws_arns": ["arn:aws:sts::123456789123:assumed-role/DevAdministrator/test_user"],
            "p_any_ip_addresses": ["1.1.1.1"],
            "p_any_trace_ids": ["ASIA3JHVJH35KB7LJHV2"],
            "p_event_time": "2022-11-23 21:06:45",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-23 21:08:20.503",
            "p_row_id": "be1f79935716d5c6faf68be41493e410",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "CloudTrail",
            "readOnly": True,
            "recipientAccountId": "123456789123",
            "requestParameters": {"map": {}},
            "sourceIPAddress": "1.1.1.1",
            "tlsDetails": {
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "us-east-1.console.aws.amazon.com",
                "tlsVersion": "TLSv1.2",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "ASIA3JHVJH35KB7LJHV2",
                "accountId": "123456789123",
                "arn": "arn:aws:sts::123456789123:assumed-role/DevAdministrator/test_user",
                "principalId": "AR0A354LKJXC87G9XC89V:test_user",
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="GetContactInformation",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "743a109a-3bce-4298-8828-114c11339119",
            "eventName": "GetContactInformation",
            "eventSource": "billingconsole.amazonaws.com",
            "eventTime": "2022-11-23 21:06:46",
            "eventType": "AwsConsoleAction",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789123"],
            "p_any_aws_arns": ["arn:aws:sts::123456789123:assumed-role/DevAdministrator/test_user"],
            "p_any_ip_addresses": ["1.1.1.1"],
            "p_any_trace_ids": ["ASIA3JHVJH35KB7LJHV2"],
            "p_event_time": "2022-11-23 21:06:46",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-11-23 21:08:20.502",
            "p_row_id": "be1f79935716d5c6faf68be4148ce410",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": True,
            "recipientAccountId": "123456789123",
            "requestParameters": {"map": {"type": "CONTACT"}},
            "sourceIPAddress": "1.1.1.1",
            "tlsDetails": {
                "cipherSuite": "ECDHE-RSA-AES128-GCM-SHA256",
                "clientProvidedHostHeader": "us-east-1.console.aws.amazon.com",
                "tlsVersion": "TLSv1.2",
            },
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
            "userIdentity": {
                "accessKeyId": "ASIA3JHVJH35KB7LJHV2",
                "accountId": "123456789123",
                "arn": "arn:aws:sts::123456789123:assumed-role/DevAdministrator/test_user",
                "principalId": "AR0A354LKJXC87G9XC89V:test_user",
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailAccountDiscovery(Rule):
    default_description = "Adversaries may attempt to get a listing of accounts on a system or within an environment. This information can help adversaries determine which accounts exist to aid in follow-on behavior."
    display_name = "AWS CloudTrail Account Discovery"
    default_reference = "https://attack.mitre.org/techniques/T1087/"
    reports = {"MITRE ATT&CK": ["TA0007:T1087"]}
    default_severity = Severity.INFO
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.CloudTrail.Account.Discovery-prototype"
    tests = aws_cloud_trail_account_discovery_tests
    DISCOVERY_EVENTS = [
        "GetAlternateContact",
        "GetContactInformation",
        "PutAlternateContact",
        "PutContactInformation",
        "DescribeAccount",
    ]

    def rule(self, event):
        return event.get("eventName") in self.DISCOVERY_EVENTS

    def title(self, event):
        return f"User [{deep_get(event, 'userIdentity', 'arn')}]performed a [{event.get('eventName')}] action in AWS account [{event.get('recipientAccountId')}]."



"""
Panther Analysis Tool is a command line interface for writing,
testing, and packaging policies/rules.
Copyright (C) 2020 Panther Labs Inc

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import argparse
import re
from typing import Any, Callable, Tuple

from pypanther import display
from pypanther.vendor.panther_analysis_tool.backend.client import Client as BackendClient
from pypanther.vendor.panther_analysis_tool.backend.public_api_client import PublicAPIClient, PublicAPIClientOptions
from pypanther.vendor.panther_analysis_tool.constants import PANTHER_USER_ID

UNKNOWN_VERSION = "unknown"


class BackendNotFoundException(Exception):
    pass


def func_with_backend(
    func: Callable[[BackendClient, argparse.Namespace], Any],
) -> Callable[[argparse.Namespace], Tuple[int, str]]:
    return lambda args: func(get_backend(args), args)


def get_backend(args: argparse.Namespace) -> BackendClient:
    if not args.api_token:
        raise BackendNotFoundException("API token is required")

    verbose = args.verbose if hasattr(args, "verbose") else False
    output_type = args.output if hasattr(args, "output") else display.OUTPUT_TYPE_TEXT
    return PublicAPIClient(
        PublicAPIClientOptions(
            token=args.api_token,
            user_id=PANTHER_USER_ID,
            host=args.api_host,
            verbose=verbose,
            output_type=output_type,
        ),
    )


def convert_unicode(obj: Any) -> str:
    """
    Swap unicode 4 byte strings with arbitrary numbers of leading slashes with the actual character
    e.g. \\\\u003c => <
    """
    string_to_convert = str(obj)
    return re.sub(r"\\*\\u([0-9a-f]{4})", lambda m: chr(int(m.group(1), 16)), string_to_convert)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import deserialize_administrator_log_event_description

duo_admin_action_marked_fraudulent_tests: list[RuleTest] = [
    RuleTest(
        name="marked_fraud",
        expected_result=True,
        log={
            "action": "admin_2fa_error",
            "description": '{"ip_address": "12.12.12.12", "email": "example@example.io", "factor": "push", "error": "Login request reported as fraudulent."}',
            "isotimestamp": "2022-12-14 20:11:53",
            "timestamp": "2022-12-14 20:11:53",
            "username": "John P. Admin",
        },
    ),
    RuleTest(
        name="different_admin_action",
        expected_result=False,
        log={
            "action": "admin_update",
            "description": "{}",
            "isotimestamp": "2022-12-14 20:11:53",
            "timestamp": "2022-12-14 20:11:53",
            "username": "John P. Admin",
        },
    ),
]


@panther_managed
class DUOAdminActionMarkedFraudulent(Rule):
    id = "DUO.Admin.Action.MarkedFraudulent-prototype"
    display_name = "Duo Admin Marked Push Fraudulent"
    dedup_period_minutes = 15
    log_types = [LogType.DUO_ADMINISTRATOR]
    tags = ["Duo"]
    default_severity = Severity.MEDIUM
    default_description = "A Duo push was marked fraudulent by an admin."
    default_reference = "https://duo.com/docs/adminapi#administrator-logs"
    default_runbook = "Follow up with the administrator to determine reasoning for marking fraud."
    tests = duo_admin_action_marked_fraudulent_tests

    def rule(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return event.get("action") == "admin_2fa_error" and "fraudulent" in event_description.get("error", "").lower()

    def title(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        admin_username = event.get("username", "Unknown")
        user_email = event_description.get("email", "Unknown")
        return f"Duo Admin [{admin_username}] denied due to an anomalous 2FA push for [{user_email}]"

    def alert_context(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return {
            "reason": event_description.get("error", ""),
            "reporting_admin": event.get("username", ""),
            "user": event_description.get("email", ""),
            "ip_address": event_description.get("ip_address", ""),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

asana_service_account_created_tests: list[RuleTest] = [
    RuleTest(
        name="New domain created",
        expected_result=False,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:30:26.15",
            "details": {"new_value": "test.com"},
            "event_category": "admin_settings",
            "event_type": "workspace_associated_email_domain_added",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Example IO", "resource_type": "workspace"},
        },
    ),
    RuleTest(
        name="Slack svc acct",
        expected_result=True,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@panther.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:28:18.396",
            "details": {},
            "event_category": "apps",
            "event_type": "service_account_created",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Slack Service Account", "resource_type": "user"},
        },
    ),
    RuleTest(
        name="Datadog svc acct",
        expected_result=True,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@panther.io",
                "gid": "12345",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:28:18.396",
            "details": {},
            "event_category": "apps",
            "event_type": "service_account_created",
            "gid": "12345",
            "resource": {"gid": "12345", "name": "Datadog Service Account", "resource_type": "user"},
        },
    ),
]


@panther_managed
class AsanaServiceAccountCreated(Rule):
    default_description = "An Asana service account was created by someone in your organization."
    display_name = "Asana Service Account Created"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://help.asana.com/hc/en-us/articles/14217496838427-Service-Accounts"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Service.Account.Created-prototype"
    tests = asana_service_account_created_tests

    def rule(self, event):
        return event.get("event_type", "<NO_EVENT_TYPE_FOUND>") == "service_account_created"

    def title(self, event):
        actor_email = deep_get(event, "actor", "email", default="<ACTOR_NOT_FOUND>")
        svc_acct_name = deep_get(event, "resource", "name", default="<SVC_ACCT_NAME_NOT_FOUND>")
        return f"Asana user [{actor_email}] created a new service account [{svc_acct_name}]."



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_user_suspended_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "login_success",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    RuleTest(
        name="Account Warning Not For User Suspended",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "suspicious_login ",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    RuleTest(
        name="Account Warning For Suspended User",
        expected_result=True,
        log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "account_disabled_spamming",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
]


@panther_managed
class GSuiteUserSuspended(Rule):
    id = "GSuite.UserSuspended-prototype"
    display_name = "GSuite User Suspended"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.HIGH
    default_description = "A GSuite user was suspended, the account may have been compromised by a spam network.\n"
    default_reference = "https://support.google.com/drive/answer/40695?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Investigate the behavior that got the account suspended. Verify with the user that this intended behavior. If not, the account may have been compromised.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_user_suspended_tests
    USER_SUSPENDED_EVENTS = {
        "account_disabled_generic",
        "account_disabled_spamming_through_relay",
        "account_disabled_spamming",
        "account_disabled_hijacked",
    }

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        return bool(event.get("name") in self.USER_SUSPENDED_EVENTS)

    def title(self, event):
        user = deep_get(event, "parameters", "affected_email_address")
        if not user:
            user = "<UNKNOWN_USER>"
        return f"User [{user}]'s account was disabled"



import json

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_lab_audit_password_reset_multiple_emails_tests: list[RuleTest] = [
    RuleTest(name="not a password reset", expected_result=False, log={"detail": {"custom_message": "hello world"}}),
    RuleTest(
        name="one email",
        expected_result=False,
        log={"detail": {"custom_message": "Ask for password reset", "target_details": "example@test.com"}},
    ),
    RuleTest(
        name="multiple emails",
        expected_result=True,
        log={
            "detail": {
                "custom_message": "Ask for password reset",
                "target_details": '["example@test.com", "example2@test.com"]',
            },
        },
    ),
]


@panther_managed
class GitLabAuditPasswordResetMultipleEmails(Rule):
    id = "GitLab.Audit.Password.Reset.Multiple.Emails-prototype"
    display_name = "CVE-2023-7028 - GitLab Audit Password Reset Multiple Emails"
    log_types = [LogType.GITLAB_AUDIT]
    tags = ["GitLab", "CVE-2023-7028"]
    reports = {"MITRE ATT&CK": ["TA0001:T1195", "TA0001:T1190", "TA0003:T1098"]}
    default_severity = Severity.HIGH
    default_description = "Attackers are exploiting a Critical (CVSS 10.0) GitLab vulnerability in which user account password reset emails could be delivered to an unverified email address."
    default_reference = "https://about.gitlab.com/releases/2024/01/11/critical-security-release-gitlab-16-7-2-released/"
    tests = git_lab_audit_password_reset_multiple_emails_tests

    def rule(self, event):
        custom_message = event.deep_get("detail", "custom_message", default="")
        emails_raw = event.deep_get("detail", "target_details", default="")
        if custom_message != "Ask for password reset":
            return False
        try:
            emails = json.loads(emails_raw)
        except json.decoder.JSONDecodeError:
            return False
        if len(emails) > 1:
            return True
        return False

    def title(self, event):
        emails = event.deep_get("detail", "target_details", default="")
        return f"[GitLab] Multiple password reset emails requested for {emails}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_government_backed_attack_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "actor": {"email": "homer.simpson@example.com"},
            "type": "login",
            "name": "login_success",
            "parameters": {"is_suspicious": None, "login_challenge_method": ["none"]},
        },
    ),
    RuleTest(
        name="Government Backed Attack Warning",
        expected_result=True,
        log={
            "id": {"applicationName": "login"},
            "actor": {"email": "homer.simpson@example.com"},
            "type": "login",
            "name": "gov_attack_warning",
            "parameters": {"is_suspicious": None, "login_challenge_method": ["none"]},
        },
    ),
]


@panther_managed
class GSuiteGovernmentBackedAttack(Rule):
    id = "GSuite.GovernmentBackedAttack-prototype"
    display_name = "GSuite Government Backed Attack"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.CRITICAL
    default_description = "GSuite reported that it detected a government backed attack against your account.\n"
    default_reference = "https://support.google.com/a/answer/9007870?hl=en"
    default_runbook = "Followup with GSuite support for more details.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_government_backed_attack_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        return bool(event.get("name") == "gov_attack_warning")

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_EMAIL>')}] may have been targeted by a government attack"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_duo_helpers import deserialize_administrator_log_event_description, duo_alert_context

duo_admin_new_admin_api_app_integration_tests: list[RuleTest] = [
    RuleTest(
        name="Admin API Integration Created",
        expected_result=True,
        log={
            "action": "integration_create",
            "description": '{"greeting": "", "notes": "", "offline_auth_enabled": 0, "offline_max_days": 0, "offline_max_attempts": 0, "type": "Admin API", "raw_type": "adminapi", "name": "Admin API", "self_service_allowed": false, "username_normalization_policy": "None", "missing_web_referer_policy": "deny", "networks_for_api_access": "", "group_access": ""}',
            "isotimestamp": "2021-11-30 17:15:33",
            "object": "Admin API",
            "timestamp": "2021-11-30 17:15:33",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Non Admin API Integration",
        expected_result=False,
        log={
            "action": "integration_create",
            "description": '{"greeting": "", "notes": "", "offline_auth_enabled": 0, "offline_max_days": 0, "offline_max_attempts": 0, "type": "1Password", "raw_type": "1password", "name": "1Password", "self_service_allowed": false, "username_normalization_policy": "None", "missing_web_referer_policy": "deny", "networks_for_api_access": "", "group_access": ""}',
            "isotimestamp": "2021-11-30 17:11:51",
            "object": "1Password",
            "timestamp": "2021-11-30 17:11:51",
            "username": "Homer Simpson",
        },
    ),
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "action": "user_update",
            "description": '{"phones": ""}',
            "isotimestamp": "2021-07-02 18:31:56",
            "object": "homer.simpson@simpsons.io",
            "timestamp": "2021-07-02 18:31:56",
            "username": "Homer Simpson",
        },
    ),
]


@panther_managed
class DuoAdminNewAdminAPIAppIntegration(Rule):
    default_description = "Identifies creation of new Admin API integrations for Duo."
    display_name = "Duo Admin New Admin API App Integration"
    default_reference = "https://duo.com/docs/adminapi#overview"
    default_severity = Severity.HIGH
    log_types = [LogType.DUO_ADMINISTRATOR]
    id = "Duo.Admin.New.Admin.API.App.Integration-prototype"
    tests = duo_admin_new_admin_api_app_integration_tests

    def rule(self, event):
        if event.get("action") == "integration_create":
            description = deserialize_administrator_log_event_description(event)
            integration_type = description.get("type")
            return integration_type == "Admin API"
        return False

    def title(self, event):
        return f"Duo: [{event.get('username', '<username_not_found>')}] created a new Admin API integration to [{event.get('object', '<object_not_found>')}]"

    def alert_context(self, event):
        return duo_alert_context(event)



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

okta_global_mfa_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="MFA Disabled",
        expected_result=True,
        log={
            "published": "2022-03-22 14:21:53.225",
            "eventType": "system.mfa.factor.deactivate",
            "version": "0",
            "severity": "HIGH",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                },
                "zone": "null",
            },
            "p_log_type": "Okta.SystemLog",
        },
    ),
    RuleTest(
        name="Login Event",
        expected_result=False,
        log={
            "published": "2022-03-22 14:21:53.225",
            "eventType": "user.session.start",
            "version": "0",
            "severity": "INFO",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                },
                "zone": "null",
            },
            "p_log_type": "Okta.SystemLog",
        },
    ),
]


@panther_managed
class OktaGlobalMFADisabled(Rule):
    id = "Okta.Global.MFA.Disabled-prototype"
    display_name = "Okta MFA Globally Disabled"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "DataModel", "Okta", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_severity = Severity.HIGH
    default_description = "An admin user has disabled the MFA requirement for your Okta account"
    default_reference = (
        "https://help.okta.com/oie/en-us/content/topics/identity-engine/authenticators/about-authenticators.htm"
    )
    default_runbook = "Contact Admin to ensure this was sanctioned activity"
    dedup_period_minutes = 15
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    tests = okta_global_mfa_disabled_tests

    def rule(self, event):
        return event.udm("event_type") == event_type.ADMIN_MFA_DISABLED

    def title(self, event):
        return f"Okta System-wide MFA Disabled by Admin User {event.udm('actor_user')}"

    def alert_context(self, event):
        context = {"user": event.udm("actor_user"), "ip": event.udm("source_ip"), "event": event.get("eventType")}
        return context
